# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from telemetry_harness.testcase import TelemetryTestCase


class TestShutdownPingsSucced(TelemetryTestCase):
    """Test Firefox shutdown pings."""

    def test_shutdown_pings_succeed(self):
        """Test that known Firefox shutdown pings are received."""

        ping_types = (
            "event",
            "first-shutdown",
            "main",
            "new-profile",
        )

        # We don't need the browser after this, but the harness expects a
        # running browser to clean up, so we `restart_browser` rather than
        # `quit_browser`.
        pings = self.wait_for_pings(
            self.restart_browser, lambda p: p["type"] in ping_types, len(ping_types)
        )

        self.assertEqual(len(pings), len(ping_types))
        self.assertEqual(set(ping_types), set(p["type"] for p in pings))



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import shutil
import subprocess
import sys
from pathlib import Path

import mozunit
from buildconfig import topsrcdir
from mach.requirements import MachEnvRequirements


def _resolve_command_site_names():
    site_names = []
    for child in (Path(topsrcdir) / "python" / "sites").iterdir():
        if not child.is_file():
            continue

        if child.name == "mach.txt":
            continue

        site_names.append(child.stem)
    return site_names


def _requirement_definition_to_pip_format(site_name, cache, is_mach_or_build_env):
    """Convert from parsed requirements object to pip-consumable format"""
    requirements_path = Path(topsrcdir) / "python" / "sites" / f"{site_name}.txt"
    requirements = MachEnvRequirements.from_requirements_definition(
        topsrcdir, False, not is_mach_or_build_env, requirements_path
    )

    lines = []
    for pypi in (
        requirements.pypi_requirements + requirements.pypi_optional_requirements
    ):
        lines.append(str(pypi.requirement))

    for vendored in requirements.vendored_requirements:
        lines.append(str(cache.package_for_vendor_dir(Path(vendored.path))))

    for pth in requirements.pth_requirements:
        path = Path(pth.path)

        if "third_party" not in (p.name for p in path.parents):
            continue

        for child in path.iterdir():
            if child.name.endswith(".dist-info"):
                raise Exception(
                    f'In {requirements_path}, the "pth:" pointing to "{path}" has a '
                    '".dist-info" file.\n'
                    'Perhaps it should change to start with "vendored:" instead of '
                    '"pth:".'
                )
            if child.name.endswith(".egg-info"):
                raise Exception(
                    f'In {requirements_path}, the "pth:" pointing to "{path}" has an '
                    '".egg-info" file.\n'
                    'Perhaps it should change to start with "vendored:" instead of '
                    '"pth:".'
                )

    return "\n".join(lines)


class PackageCache:
    def __init__(self, storage_dir: Path):
        self._cache = {}
        self._storage_dir = storage_dir

    def package_for_vendor_dir(self, vendor_path: Path):
        if vendor_path in self._cache:
            return self._cache[vendor_path]

        if not any((p for p in vendor_path.iterdir() if p.name.endswith(".dist-info"))):
            # This vendored package is not a wheel. It may be a source package (with
            # a setup.py), or just some Python code that was manually copied into the
            # tree. If it's a source package, the setup.py file may be up a few levels
            # from the referenced Python module path.
            package_dir = vendor_path
            while True:
                if (package_dir / "setup.py").exists():
                    break
                elif package_dir.parent == package_dir:
                    raise Exception(
                        f'Package "{vendor_path}" is not a wheel and does not have a '
                        'setup.py file. Perhaps it should be "pth:" instead of '
                        '"vendored:"?'
                    )
                package_dir = package_dir.parent

            self._cache[vendor_path] = package_dir
            return package_dir

        # Pip requires that wheels have a version number in their name, even if
        # it ignores it. We should parse out the version and put it in here
        # so that failure debugging is easier, but that's non-trivial work.
        # So, this "0" satisfies pip's naming requirement while being relatively
        # obvious that it's a placeholder.
        output_path = self._storage_dir / f"{vendor_path.name}-0-py3-none-any"
        shutil.make_archive(str(output_path), "zip", vendor_path)

        whl_path = output_path.parent / (output_path.name + ".whl")
        (output_path.parent / (output_path.name + ".zip")).rename(whl_path)
        self._cache[vendor_path] = whl_path

        return whl_path


def test_sites_compatible(tmpdir: str):
    command_site_names = _resolve_command_site_names()
    work_dir = Path(tmpdir)
    cache = PackageCache(work_dir)
    mach_requirements = _requirement_definition_to_pip_format("mach", cache, True)

    # Create virtualenv to try to install all dependencies into.
    subprocess.check_call(
        [
            sys.executable,
            str(
                Path(topsrcdir)
                / "third_party"
                / "python"
                / "virtualenv"
                / "virtualenv.py"
            ),
            "--no-download",
            str(work_dir / "env"),
        ]
    )

    for name in command_site_names:
        print(f'Checking compatibility of "{name}" site')
        command_requirements = _requirement_definition_to_pip_format(
            name, cache, name == "build"
        )
        with open(work_dir / "requirements.txt", "w") as requirements_txt:
            requirements_txt.write(mach_requirements)
            requirements_txt.write("\n")
            requirements_txt.write(command_requirements)

        # Attempt to install combined set of dependencies (global Mach + current
        # command)
        subprocess.check_call(
            [
                str(work_dir / "env" / "bin" / "pip"),
                "install",
                "-r",
                str(work_dir / "requirements.txt"),
            ],
            cwd=topsrcdir,
        )


if __name__ == "__main__":
    mozunit.main()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from __future__ import absolute_import

import copy

import requests

from marionette_harness import MarionetteTestCase


class TestCommandLineArguments(MarionetteTestCase):
    def setUp(self):
        super(TestCommandLineArguments, self).setUp()

        self.orig_arguments = copy.copy(self.marionette.instance.app_args)

    def tearDown(self):
        self.marionette.instance.app_args = self.orig_arguments
        self.marionette.quit(clean=True)

        super(TestCommandLineArguments, self).tearDown()

    def is_bidi_enabled(self):
        with self.marionette.using_context("chrome"):
            bidi_enabled = self.marionette.execute_script(
                """
              const { RemoteAgent } = ChromeUtils.import(
                "chrome://remote/content/components/RemoteAgent.jsm"
              );
              return !!RemoteAgent.webDriverBiDi;
            """
            )

    def test_debugger_address_cdp_status(self):
        # By default Remote Agent is not enabled
        debugger_address = self.marionette.session_capabilities.get(
            "moz:debuggerAddress"
        )
        self.assertIsNone(debugger_address)

        # With BiDi only enabled the capability doesn't have to be returned
        self.marionette.enforce_gecko_prefs({"remote.active-protocols": 1})
        try:
            self.marionette.quit()
            self.marionette.instance.app_args.append("-remote-debugging-port")
            self.marionette.start_session()

            debugger_address = self.marionette.session_capabilities.get(
                "moz:debuggerAddress"
            )
            self.assertIsNone(debugger_address)
        finally:
            self.marionette.clear_pref("remote.active-protocols")
            self.marionette.restart()

        # With all protocols enabled the capability has to be returned
        self.marionette.quit()
        self.marionette.instance.switch_profile()
        self.marionette.start_session()

        debugger_address = self.marionette.session_capabilities.get(
            "moz:debuggerAddress"
        )

        self.assertEqual(debugger_address, "localhost:9222")
        result = requests.get(url="http://{}/json/version".format(debugger_address))
        self.assertTrue(result.ok)

    def test_websocket_url(self):
        # By default Remote Agent is not enabled
        self.assertNotIn("webSocketUrl", self.marionette.session_capabilities)

        # With BiDi not enabled the capability is still not returned
        self.marionette.enforce_gecko_prefs({"remote.active-protocols": 2})
        try:
            self.marionette.quit()
            self.marionette.instance.app_args.append("-remote-debugging-port")
            self.marionette.start_session({"webSocketUrl": True})

            self.assertNotIn("webSocketUrl", self.marionette.session_capabilities)
        finally:
            self.marionette.clear_pref("remote.active-protocols")
            self.marionette.restart()

        # With BiDi enabled the capability has to be returned
        if self.is_bidi_enabled():
            self.marionette.quit()
            self.marionette.instance.switch_profile()
            self.marionette.start_session({"webSocketUrl": True})

            session_id = self.marionette.session_id
            websocket_url = self.marionette.session_capabilities.get("webSocketUrl")

            self.assertEqual(
                websocket_url, "ws://localhost:9222/session/{}".format(session_id)
            )

    # An issue in the command line argument handling lead to open Firefox on
    # random URLs when remote-debugging-port is set to an explicit value, on macos.
    # See Bug 1724251.
    def test_start_page_about_blank(self):
        if self.is_bidi_enabled():
            self.marionette.quit()
            self.marionette.instance.app_args.append("-remote-debugging-port=0")
            self.marionette.start_session({"webSocketUrl": True})
            self.assertEqual(self.marionette.get_url(), "about:blank")

    def test_startup_timeout(self):
        try:
            self.marionette.quit()
            with self.assertRaisesRegexp(IOError, "Process killed after 0s"):
                # Use a small enough timeout which should always cause an IOError
                self.marionette.start_session(timeout=0)
        finally:
            self.marionette.start_session()
